var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var fetch_exports = {};
__export(fetch_exports, {
  default: () => fetch_default,
  fetchStream: () => fetchStream,
  setDefaults: () => setDefaults
});
module.exports = __toCommonJS(fetch_exports);
const defaults = {
  rateLimit: 0.01,
  rateLimitTimestamp: 0,
  dataPath: void 0,
  nextPath: void 0,
  qs: void 0,
  method: "GET",
  headers: {
    Accept: "application/json",
    "Accept-Encoding": "br, gzip, deflate"
  }
};
const setDefaults = (options) => {
  const headers = { ...defaults.headers, ...options.headers };
  Object.assign(defaults, options, { headers });
};
async function* fetchStream(optionsArr, { signal } = {}) {
  if (!Array.isArray(optionsArr))
    optionsArr = [optionsArr];
  const requests = [];
  for (let options of optionsArr) {
    options.headers = { ...defaults.headers, ...options.headers };
    options = { ...defaults, ...options };
    if (options.qs)
      options.url += "?" + new URLSearchParams(options.qs);
    if (options.headers.Accept.includes("application/json")) {
      requests.push(fetchJson(options, { signal }));
    } else {
      requests.push(fetchBody(options, { signal }));
    }
  }
  for (const request of requests) {
    const response = await request;
    for await (const chunk of response) {
      yield chunk;
    }
  }
}
const fetchBody = async (options, { signal }) => {
  const response = await fetchRateLimit(options, { signal });
  return response.body;
};
const nextLinkRegExp = /<(.*?)>; rel="next"/;
async function* fetchJson(options, { signal }) {
  const { dataPath, nextPath } = options;
  while (options.url) {
    const response = await fetchRateLimit(options, { signal });
    const body = await response.json();
    options.url = nextPath && pickPath(body, nextPath);
    options.url ??= parseLink(response.headers);
    const data = pickPath(body, dataPath);
    if (Array.isArray(data)) {
      for (const item of data) {
        yield item;
      }
    } else {
      yield data;
    }
  }
}
const parseLink = (headers) => {
  const link = headers.get("Link");
  if (link) {
    return link.match(nextLinkRegExp)?.[1];
  }
};
const fetchRateLimit = async (options, { signal }) => {
  options.rateLimitTimestamp ??= 0;
  const now = Date.now();
  if (now < options.rateLimitTimestamp) {
    await timeout(options.rateLimitTimestamp - now, { signal });
  }
  options.rateLimitTimestamp = now + 1e3 * options.rateLimit;
  return fetch(options, { signal });
};
const pickPath = (obj, path) => {
  if (path === "")
    return obj;
  if (!Array.isArray(path))
    path = path.split(".");
  return path.slice(0).reduce((a, b) => {
    return a[b];
  }, obj);
};
const timeout = (ms, { signal }) => {
  if (signal?.aborted) {
    return Promise.reject(new Error("Aborted", "AbortError"));
  }
  return new Promise((resolve, reject) => {
    const abortHandler = () => {
      clearTimeout(timeout2);
      reject(new Error("Aborted", "AbortError"));
    };
    if (signal)
      signal.addEventListener("abort", abortHandler);
    const timeout2 = setTimeout(() => {
      resolve();
      if (signal)
        signal.removeEventListener("abort", abortHandler);
    }, ms);
  });
};
var fetch_default = fetchStream;
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  fetchStream,
  setDefaults
});
